/**
 * The ClockDisplay class implements a digital clock display for a
 * European-style 24 hour clock. The clock shows hours, minutes and seconds. The 
 * range of the clock is 00:00:00 (midnight) to 23:59:59 (one second before 
 * midnight).
 * 
 * The clock display receives "ticks" (via the timeTick method) every second
 * and reacts by incrementing the display. This is done in the usual clock
 * fashion: the hour increments when the minutes roll over to zero and
 * the minute increments when the seconds roll over to zero.
 * 
 * @author Michael Kolling and David J. Barnes and J. Icha
 * @version 2008.03.30, 1. 4. 2013
 */
public class ClockDisplay {
    private NumberDisplay hours;
    private NumberDisplay minutes;
    private NumberDisplay seconds;
    private String displayString; // simulates the actual display

    /**
     * Constructor for ClockDisplay objects. This constructor 
     * creates a new clock set at 00:00:00.
     */
    public ClockDisplay() {
        hours = new NumberDisplay(24);
        minutes = new NumberDisplay(60);
        seconds = new NumberDisplay(60);
        updateDisplay();
    }

    /**
     * Constructor for ClockDisplay objects. This constructor
     * creates a new clock set at the time specified by the 
     * parameters.
     */
    public ClockDisplay(int hour, int minute, int second) {
        hours = new NumberDisplay(24);
        minutes = new NumberDisplay(60);
        seconds = new NumberDisplay(60);
        setTime(hour, minute, second);
    }

    /**
     * This method should get called once every second - it makes
     * the clock display go one second forward.
     */
    public void timeTick() {
        seconds.increment();
        if(seconds.getValue() == 0) {  // it just rolled over!
            minutes.increment();
            if(minutes.getValue() == 0) {  // it just rolled over!
                hours.increment();
            }
        }
        updateDisplay();
    }

    /**
     * Set the time of the display to the specified hour, minute and
     * second.
     */
    public void setTime(int hour, int minute, int second) {
        hours.setValue(hour);
        minutes.setValue(minute);
        seconds.setValue(second);
        updateDisplay();
    }

    /**
     * Return the current time of this display in the format HH:MM:SS.
     */
    public String getTime() {
        return displayString;
    }
    
    /**
     * Return the hours limit.
     */
    public int getLimitH() {
        return hours.getLimit();
    }
    
    /**
     * Return the minutes and seconds limit.
     */
    public int getLimitMaS() {
        return  minutes.getLimit();
    }
    
    /**
     * Return the array with hours, minutes and seconds.
     */
    public int[] getHMS() {
        String[] timeHMS = getTime().split(":");
        int[] result = new int[3];
        for(int i = 0; i < timeHMS.length; i++){
            result[i] = Integer.parseInt(timeHMS[i]);
        }
        return result;
    }
    /**
     * Update the internal string that represents the display.
     */
    private void updateDisplay() {
        displayString = hours.getDisplayValue() + ":"
                + minutes.getDisplayValue() + ":" + seconds.getDisplayValue();
    }
}