/**
A panel with mouse operations for adding and removing squares.
 */
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.awt.geom.*;
import javax.swing.*;

class MousePanel extends JPanel
{
    public MousePanel()
    {
        circles = new ArrayList<Ellipse2D>();
        colors = new ArrayList<Color>();
        current = null;
        setBackground(Color.yellow);

        addMouseListener(new MouseHandler());
        addMouseMotionListener(new MouseMotionHandler());
    }

    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D) g;

        // draw all squares
        for (int i = 0; i < circles.size(); i++){          
            g2.setColor(colors.get(i));
            g2.fill(circles.get(i));
        }
    }

    private Color getRandomColor()
    {
        return new Color(rand.nextInt(256), rand.nextInt(256), rand.nextInt(256));
    }

    /**
    Finds the first square containing a point.
    @param p a point
    @return the first square that contains p
     */
    public Ellipse2D find(Point2D p)
    {
        for (Ellipse2D r : circles)
        {
            if (r.contains(p)) return r;
        }
        return null;
    }

    /**
    Adds a square to the collection.
    @param p the center of the square
     */
    public void add(Point2D p)
    {
        double x = p.getX();
        double y = p.getY();

        current = new Ellipse2D.Double(
            x - SIDELENGTH / 2,
            y - SIDELENGTH / 2,
            SIDELENGTH,
            SIDELENGTH);
        circles.add(current);
        colors.add(getRandomColor());
        repaint();
    }

    /**
    Removes a square from the collection.
    @param s the square to remove
     */
    public void remove(Ellipse2D s)
    {
        if (s == null) return;
        if (s == current) current = null;
        int index = circles.indexOf(s);
        circles.remove(s);
        colors.remove(index);
        repaint();
    }
    
    public void clear()
    {
        circles.clear();
        colors.clear();
         repaint();
    }

    private static final int SIDELENGTH = 15;
    private static final Random rand = new Random();
    private ArrayList<Ellipse2D> circles;
    private ArrayList<Color> colors;
    private Ellipse2D current;
    // the square containing the mouse cursor

    private class MouseHandler extends MouseAdapter
    {
        public void mousePressed(MouseEvent event)
        {
            // add a new square if the cursor isn't inside a square
            current = find(event.getPoint());
            if (current == null)
                add(event.getPoint());
        }

        public void mouseClicked(MouseEvent event)
        {
            // remove the current square if double clicked
            current = find(event.getPoint());
            if (current != null && event.getClickCount() >= 2)
                remove(current);
        }
    }

    private class MouseMotionHandler
    implements MouseMotionListener
    {
        public void mouseMoved(MouseEvent event)
        {
            // set the mouse cursor to cross hairs if it is inside
            // a rectangle

            if (find(event.getPoint()) == null)
                setCursor(Cursor.getDefaultCursor());
            else
                setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
        }

        public void mouseDragged(MouseEvent event)
        {
            if (current != null)
            {
                int x = event.getX();
                int y = event.getY();

                // drag the current rectangle to center it at (x, y)
                current.setFrame(
                    x - SIDELENGTH / 2,
                    y - SIDELENGTH / 2,
                    SIDELENGTH,
                    SIDELENGTH);
                repaint();
            }
        }
    }
}